﻿using System.Collections.Generic;
using NUnit.Framework;
using Tile16_Decoupled;

namespace Test_Tile16_Decoupled
{
    [TestFixture]
    public class TestBoard
    {
        [Test]
        public void TestCreateBoard()
        {
            Board board = new Board(1,1);
            Assert.That(board.GetWidth(), Is.EqualTo(1));
            Assert.That(board.GetHeight(), Is.EqualTo(1));

            board  = new Board(5,6);
            Assert.That(board.GetWidth(), Is.EqualTo(5));
            Assert.That(board.GetHeight(), Is.EqualTo(6));

        }

        [Test]
        public void CreateBoardWithInvalidDimensions()
        {
            Assert.Throws<InvalidBoardDimensionsException>(() => new Board(0, 0));
        }

        [TestFixture]
        public class Board4X4Tests
        {
            private Board board;
            [SetUp]
            public void Setup()
            {
                board = new Board(4, 4);
            }
            [Test]
            public void AddTileToBoard()
            {
                board.AddTile(new Tile(), new Point(1, 1));
                board.AddTile(new Tile(), new Point(3, 3));
                Assert.That(board.TileExists(new Point(1, 1)), Is.True);
                Assert.That(board.TileExists(new Point(3, 3)), Is.True);

            }

            [Test]
            public void AddTileToLocationWhereTileExists()
            {
                board.AddTile(new Tile(), new Point(1, 1));
                board.AddTile(new Tile(), new Point(3, 3));
                Assert.Throws<BoardLocationOccupiedException>(() => board.AddTile(new Tile(), new Point(1, 1)));
                Assert.Throws<BoardLocationOccupiedException>(() => board.AddTile(new Tile(), new Point(3, 3)));

            }


            [Test]
            public void AddTileToBoardWithInvalidCoord()
            {
                Assert.Throws<InvalidBoardCoordinatesExcpetion>(() => board.AddTile(new Tile(), new Point(-1, -1)));
                Assert.Throws<InvalidBoardCoordinatesExcpetion>(() => board.AddTile(new Tile(), new Point(4, 4)));

            }

            [Test]
            public void TestTileNotExists()
            {
                board.AddTile(new Tile(), new Point(3, 3));
                Assert.That(board.TileNotExists(new Point(1, 1)), Is.True);
                Assert.That(board.TileNotExists(new Point(3, 3)), Is.False);
            }

            [Test]
            public void AddThenGetTileWithSameValue()
            {
                var tileLocation = new Point(3, 3);
                board.AddTile(new Tile(5), tileLocation);
                Assert.That(new Tile(5), Is.EqualTo(board.GetTile(tileLocation)));
            }

            [Test]
            public void GetTileAtInvalidLocation()
            {
                Assert.Throws<InvalidBoardCoordinatesExcpetion>(() => board.GetTile(new Point(-1, -1)));
                Assert.Throws<InvalidBoardCoordinatesExcpetion>(() => board.GetTile(new Point(4, 4)));
                Assert.Throws<BoardLocationNotOccupiedException>(() => board.GetTile(new Point(0, 0)));
            }

            [Test]
            public void AddThenRemoveTileFromBoard()
            {
                board.AddTile(new Tile(5), new Point(1,1));
                Assert.That(board.TileExists(new Point(1, 1)), Is.True);
                var tile = board.PopTile(new Point(1, 1));
                Assert.That(board.TileNotExists(new Point(1, 1)), Is.True);
                Assert.That(tile, Is.EqualTo(new Tile(5)));
            }

            [Test]
            public void AddingTilesShouldCreateNewTileCache()
            {
                board.AddTile(new Tile(), new Point(0,0));
                Assert.That(board.PopNewlyCreatedTiles(), Is.EqualTo(new Dictionary<Point, Tile>() { {new Point(0,0), new Tile() } }));
                board.AddTile(new Tile(), new Point(1, 1));
                Assert.That(board.PopNewlyCreatedTiles(), Is.EqualTo(new Dictionary<Point, Tile>() { { new Point(1, 1), new Tile() } }));
                board.AddTile(new Tile(), new Point(0, 1));
                board.AddTile(new Tile(), new Point(0, 2));
                board.AddTile(new Tile(), new Point(0, 3));
                Assert.That(board.PopNewlyCreatedTiles(), Is.EqualTo(
                    new Dictionary<Point, Tile>()
                    {
                        { new Point(0, 1), new Tile() },
                        { new Point(0, 2), new Tile() },
                        { new Point(0, 3), new Tile() }
                    }));
            }

            [Test]
            public void MoveTileTestToEmptyGrid()
            {
                board.AddTile(new Tile(), new Point(0, 0));
                Assert.That(board.TileExists(new Point(0, 0)), Is.True);
                board.MoveTile(new Point(0, 0), new Point(1, 1));
                Assert.That(board.TileExists(new Point(0, 0)), Is.False);
                Assert.That(board.TileExists(new Point(1, 1)), Is.True);
            }
        } 
        
    }

}