﻿using NUnit.Framework;
using Tile16_Decoupled;

namespace Test_Tile16_Decoupled
{
    [TestFixture]
    public class TestWaveController : WaveController
    {
        [SetUp]
        public void Setup()
        {
            Board newBoard = new Board(4, 4);
            SetBoard(newBoard);
            winConditions = new WinConditions();
            winConditions.SetBoard(newBoard);
            winConditions.Reset();
        }

        [Test]
        public void TestInitializeGame()
        {
            InitializeNewGame();
            AssertGameNotOverAndNotWon();
        }

        private void AssertGameNotOverAndNotWon()
        {
            Assert.That(winConditions.GameWon, Is.False);
            Assert.That(winConditions.GameOver, Is.False);
        }


        [Test]
        public void SetWinningTileValueTo1ShouldWinAfterOneWave()
        {
            StartGameWithWinningValue(1);
            NextWave();
            Assert.That(winConditions.GameWon, Is.True);

        }

        private void StartGameWithWinningValue(int tileValue)
        {
            winConditions.WinningTileValue = tileValue;
            InitializeNewGame();
        }
        
        [Test]
        public void NoMovementsShouldEndInGameOverAfter5Waves()
        {
            StartGameWithWinningValue(2);
            for (int i = 0; i < 4; i++)
            {
                NextWave();
                AssertGameNotOverAndNotWon();
            }
            NextWave();
            Assert.That(winConditions.GameWon, Is.False);
            Assert.That(winConditions.GameOver, Is.True);
        }

        protected override int GenerateSpawnCount()
        {
            return 4;
        }
        
    }
    
}