﻿using NUnit.Framework;
using Tile16_Decoupled;
using Tile16_Unity_Testable;
using UnityEngine;

namespace Test_Tile16_Unity_Testable
{
    [TestFixture]
    public class TestTileCreator
    {
        [Test]
        public void TestTileCreatorMediator()
        {
            var mockTileCreatorMediator = new MockTileCreatorMediator();
            var tileCreator = new TileCreator(mockTileCreatorMediator);
            Point point = new Point(0,0);
            Tile tile = new Tile(7);
            tileCreator.Create(point, tile);
            Assert.That(mockTileCreatorMediator.IsInstantiateTileCalled, Is.True);
            Assert.That(mockTileCreatorMediator.IsAddToTileControllerDictionaryCalled, Is.True);
            Assert.That(mockTileCreatorMediator.Position, Is.EqualTo(new Vector2(50.0f, 50.0f)));
            Assert.That(mockTileCreatorMediator.ValueText, Is.EqualTo("7"));
            Assert.That(mockTileCreatorMediator.TileColor, Is.EqualTo(new Color32(0, 245, 255, 255)));

        }
    }

    public class MockTileCreatorMediator : TileCreatorMediator
    {

        public bool IsInstantiateTileCalled { get; set; } = false;
        public bool IsAddToTileControllerDictionaryCalled { get; set; } = false;
        public bool IsValueTextSet = false;
        public bool IsPositionSet = true;

        public Vector2 Position { get; set; }
        public string ValueText { get; set; }
        public Color32 TileColor { get; set; }

        public float GetTileWidth()
        {
            return 100.0f;
        }

        public Vector2 GetGridLowerLeftCorner()
        {
            return new Vector2(0.0f, 0.0f);
        }

        public void InstantiateTile()
        {
            IsInstantiateTileCalled = true;
        }

        public void AddTileToTileControllerDictionary(Tile tile, Point pos)
        {
            IsAddToTileControllerDictionaryCalled = true;
        }

        public void SetNewTile(GameObject newTile)
        {
            
        }

        public void SetValueText(string valueText)
        {
            ValueText = valueText;
        }

        public void SetTilePosition(Vector2 position)
        {
            Position = position;
        }

        public void SetTileColor(Color32 tileColor)
        {
            TileColor = tileColor;
        }
    }
}