﻿using NUnit.Framework;
using Tile16_Unity_Testable;
using UnityEngine;

namespace Test_Tile16_Unity_Testable
{
    [TestFixture]
    public class TestTileSnapper
    {
        [Test]
        public void CreateSnappingGrid()
        {
            var snappingGrid = new TileSnappingGrid();
            snappingGrid.SetTileSideLength(100);
            snappingGrid.InitializeGridWithDimensions(4, 4);
            Assert.That(snappingGrid.GetWidth(), Is.EqualTo(400));
            Assert.That(snappingGrid.GetLength(), Is.EqualTo(400));
        }

        [TestFixture]
        public class SnappingPositionTests
        {
            private TileSnappingGrid snappingGrid;

            [SetUp]
            public void Setup()
            {
                snappingGrid = new TileSnappingGrid();
                snappingGrid.SetTileSideLength(100);
            }

            [Test]
            public void SnapTileIn1X1Grid()
            {
                snappingGrid.InitializeGridWithDimensions(1, 1);
                SnapFromToPosition(new Vector2(0, 0), new Vector2(50, 50));
                SnapFromToPosition(new Vector2(-1, -1), new Vector2(-1,-1));
                SnapFromToPosition(new Vector2(99, 99), new Vector2(50,50));
                SnapFromToPosition(new Vector2(100, 100), new Vector2(100, 100));
            }

            [Test]
            public void SnapTileIn2X1Grid()
            {
                snappingGrid.InitializeGridWithDimensions(2, 1);
                SnapFromToPosition(new Vector2(100, 0), new Vector2(150, 50));
                SnapFromToPosition(new Vector2(100, 99), new Vector2(150, 50));
                SnapFromToPosition(new Vector2(199, 0), new Vector2(150, 50));
                SnapFromToPosition(new Vector2(199, 99), new Vector2(150, 50));

                SnapFromToPosition(new Vector2(200, 0), new Vector2(200, 0));
                SnapFromToPosition(new Vector2(100, 100), new Vector2(100, 100));
                
            }

            [Test]
            public void SnapTilein1x2Grid()
            {
                snappingGrid.InitializeGridWithDimensions(1, 2);
                SnapFromToPosition(new Vector2(0, 100), new Vector2(50, 150));
                SnapFromToPosition(new Vector2(0, 199), new Vector2(50, 150));
                SnapFromToPosition(new Vector2(99, 100), new Vector2(50, 150));
                SnapFromToPosition(new Vector2(99, 199), new Vector2(50, 150));

                SnapFromToPosition(new Vector2(0, 200), new Vector2(0, 200));
                SnapFromToPosition(new Vector2(99, 200), new Vector2(99, 200));
            }

            [Test]
            public void SnapTilein4x4Grid()
            {
                snappingGrid.InitializeGridWithDimensions(4, 4);
                SnapFromToPosition(new Vector2(100, 100), new Vector2(150, 150));
                SnapFromToPosition(new Vector2(199, 199), new Vector2(150, 150));
                SnapFromToPosition(new Vector2(200, 200), new Vector2(250, 250));
                SnapFromToPosition(new Vector2(399, 399), new Vector2(350, 350));
                SnapFromToPosition(new Vector2(400, 400), new Vector2(400, 400));
            }
            [Ignore]
            [Test]
            public void SnapTileFloatingPoint1x1Grid()
            {
                snappingGrid.InitializeGridWithDimensions(1, 1);
                SnapFromToPosition(new Vector2(100, 100), new Vector2(150, 150));
            }

            private void SnapFromToPosition(Vector2 pos, Vector2 snapPos)
            {
                Assert.That(snappingGrid.GetSnappingPosition(pos), Is.EqualTo(snapPos));
            }
        }
        

    }
}
