﻿using Tile16_Decoupled;
using Tile16_Unity_Testable;
using UnityEngine;


public class GameControllerBehavior : MonoBehaviour
{
    private GameController gameController;
    private GameControllerBehaviorMediator mediator;
    public GameObject LoseSequence;
    public GameObject WinSequence;
    public Board GameBoard;
    public BoardUpdaterBehavior UnityBoardUpdater;
    private int winningTileValue = 16;
    private WinConditions winConditions;
    private WaveController waveController;

    void Start()
    {
        InitializeGameControllerWithMediator();
        NewGame();
    }

    private void InitializeGameControllerWithMediator()
    {
        gameController = new GameController();
        mediator = new GameControllerBehaviorMediator(this);
        gameController.SetMediator(mediator);
        TileControllerBehavior.CurBoardUpdater = UnityBoardUpdater;
        mediator.SetBoardUpdater(UnityBoardUpdater);
        mediator.SetLoseSequence(LoseSequence);
        mediator.SetWinSequence(WinSequence);
    }

    public void NewGame()
    {
        gameController.NewGame();
        SetupGameBoardForOtherBehaviors();
        SetupWinCondition();
        SetupWaveController();
    }

    private void SetupGameBoardForOtherBehaviors()
    {
        GameBoard = gameController.GameBoard;
        UnityBoardUpdater.GameBoard = GameBoard;
    }

    private void SetupWaveController()
    {
        waveController = new WaveController();
        waveController.SetBoard(GameBoard);
        waveController.SetWinConditions(winConditions);
        waveController.InitializeNewGame();
        gameController.SetWaveController(waveController);
    }

    private void SetupWinCondition()
    {
        winConditions = new WinConditions();
        winConditions.SetBoard(GameBoard);
        winConditions.WinningTileValue = winningTileValue;
        gameController.SetWinCondition(winConditions);
    }

    void Update()
    {
        gameController.Update();
    }

    public void ReturnToMenu()
    {
        Application.LoadLevel(0);
    }

}
