﻿using Tile16_Decoupled;
using Tile16_Unity_Testable;
using UnityEngine;


public class TileControllerBehavior : MonoBehaviour
{
    public static BoardUpdaterBehavior CurBoardUpdater;

    private TileController tileController;
    private TileControllerBehaviorMediator mediator;

    private void Awake()
    {
        tileController = new TileController();
        mediator = new TileControllerBehaviorMediator(this);
        mediator.SetTileInputController(tileController);
        mediator.UnityBoardUpdater = CurBoardUpdater;
        tileController.SetMediator(mediator);
    }

    public void InitTile(Tile tile, Point pos)
    {
        tileController.InitTile(tile, pos);
    }


    private void FixedUpdate()
    {
        mediator.MoveMouse();
    }


    private void OnMouseDown()
    {
        mediator.MouseDown();
    }

    private void OnCollisionEnter2D(Collision2D collision)
    {
        if (tileController.Destroyed) return;
        var target = collision.gameObject;
        if (target.tag != "Tile") return;
        mediator.SetCollidedObject(target);
        tileController.Combine(target.GetComponent<TileControllerBehavior>().tileController);
    }

    private void OnMouseUp()
    {
        mediator.MouseUp();
    }

    public static void ClearAllTiles()
    {
        TileController.ClearAllTiles();
    }
}
