using Tile16_Unity_Testable;
using UnityEngine;

public class TileControllerBehaviorMediator : TileControllerMediator
{
    //Unity components
    private Animator animator;
    private BoxCollider2D boxCollider;
    private Rigidbody2D ridgidBody;
    private SpriteRenderer spriteRenderer;
    private AudioSource sfxPickUp;
    private AudioSource sfxDrop;
    private AudioSource sfxCombine;
    private TextMesh tileLabelMesh;

    private MonoBehaviour tileControllerBehaviour;
    private TileController tileController;
    private GameObject collidedTo;
    public BoardUpdaterBehavior UnityBoardUpdater;

    public TileControllerBehaviorMediator(MonoBehaviour tileController)
    {
        this.tileControllerBehaviour = tileController;
        InitializeComponents();
    }

    public void SetTileInputController(TileController tileController)
    {
        this.tileController = tileController;
    }

    private void InitializeComponents()
    {
        animator = tileControllerBehaviour.GetComponent<Animator>();
        boxCollider = tileControllerBehaviour.GetComponent<BoxCollider2D>();
        ridgidBody = tileControllerBehaviour.GetComponent<Rigidbody2D>();
        spriteRenderer = tileControllerBehaviour.GetComponent<SpriteRenderer>();
        tileLabelMesh = tileControllerBehaviour.transform.GetChild(0).gameObject.GetComponent<TextMesh>();
        var soundSources = tileControllerBehaviour.GetComponents<AudioSource>();
        sfxPickUp = soundSources[0];
        sfxDrop = soundSources[1];
        sfxCombine = soundSources[2];
    }

    //Inputs

    public void MouseDown()
    {
        tileController.PickUp();    
    }

    public void MoveMouse()
    {
        if (tileController == null) return;
        Vector3 pos = Input.mousePosition;
        pos.z = tileControllerBehaviour.transform.position.z - Camera.main.transform.position.z;
        var mouseWorldPos = Camera.main.ScreenToWorldPoint(pos);
        tileController.FollowMouse(mouseWorldPos);
    }

    public void MouseUp()
    {
        tileController.Drop();
    }

    public void SetCollidedObject(GameObject to)
    {
        collidedTo = to;
    }

    public void DestroyCollisionObjects()
    {   
        UnityEngine.Object.Destroy(tileControllerBehaviour.gameObject, sfxCombine.clip.length);
        UnityEngine.Object.Destroy(collidedTo);
    }

    public void DestroySelf()
    {
        if (tileControllerBehaviour != null)
            UnityEngine.Object.Destroy(tileControllerBehaviour.gameObject);
    }

    public void KinematicSet()
    {
        ridgidBody.isKinematic = true;
    }

    public Vector2 GetPosition()
    {
        return tileControllerBehaviour.transform.position;
    }

    public Vector2 GetVelocity()
    {
        return ridgidBody.velocity;
    }

    public float GetBoundSizeX()
    {
        return spriteRenderer.bounds.size.x;
    }

    public void SetRendererInactive()
    {
        spriteRenderer.enabled = false;
        tileLabelMesh.GetComponent<Renderer>().enabled = false;
    }

    public void PlayCombineSfx()
    {
        sfxCombine.Play();
    }

    public void PlaySpawnAnimation()
    {
        animator.SetTrigger("Spawned");
    }

    public void SetColliderSize(Vector2 vector2)
    {
        boxCollider.size = vector2;
    }

    public void KinematicUnset()
    {
        ridgidBody.isKinematic = false;
    }

    public void PlayDropAnimation()
    {
        animator.SetTrigger("Dropped");
    }

    public void PlayDropSfx()
    {
        sfxDrop.Play();
    }

    public void PlayPickUpAnimation()
    {
        animator.SetTrigger("PickedUp");
    }

    public void PlayPickUpSfx()
    {
        sfxPickUp.Play();
    }
    
    public void SetVelocity(Vector2 velocity)
    {
        ridgidBody.velocity = velocity;
    }

    public void SetPosition(Vector2 vector2)
    {
        tileControllerBehaviour.transform.position = vector2;
    }

    public float GetScaleX()
    {
        return tileControllerBehaviour.transform.localScale.x;
    }

    public void UpdateBoard()
    {
        UnityBoardUpdater.UpdateBoard();
    }
}