﻿using System.Runtime.Remoting.Messaging;

namespace Tile16_Decoupled
{
    public class TileMover
    {
        private Board board;
        private BoardPathFinder pathFinder;

        public TileMover(Board board)
        {
            this.board = board;
            pathFinder = new NullPathFinder();
        }

        public void Move(Point from, Point to)
        {
            if (from.Equals(to) || !pathFinder.PathExists(from,to)) return;
            if (board.TileNotExists(to))
                board.MoveTile(from, to);
            else
                CombineTiles(from, to);
        }

        private void CombineTiles(Point from, Point to)
        {
            var fromTile = board.GetTile(from);
            var toTile = board.GetTile(to);
            if (!toTile.Equals(fromTile)) return;
            toTile = new Tile(toTile.GetValue() + 1);
            board.PopTile(from);
            board.PopTile(to);
            board.AddTile(toTile, to);
        }

        public void SetPathFinder(BoardPathFinder pathFinder)
        {  
            this.pathFinder = pathFinder ?? new NullPathFinder();
        }
    }
}