﻿using System;

namespace Tile16_Decoupled
{
    /*
        The spawning order is:
        1. if no tiles on board, must spawn a pair with same value
        2. if 1 tile on board, must spawn at least one with same value
        3. if >1 tile on board and board has no repeats, must generate at least 1 with repeat
        4. if >1 tile on board and has repeats, generate within bounded values
    */
    public class TileSpawner
    {
        private Board board;
        private int maxTileValue = 1;
        private int minTileValue = 1;
        private bool hasDuplicate;
        private Random rand = new Random();

        public TileSpawner(Board board)
        {
            this.board = board;
        }

        public void Spawn(int count)
        {
            count = count > board.NumberOfUnoccupiedSpace() ? board.NumberOfUnoccupiedSpace() : count;
            hasDuplicate = false;
            SpawnMultipleRandomTiles(count);
        }

        private void SpawnMultipleRandomTiles(int count)
        {
            for (int i = 0; i < count; i++)
            {
                hasDuplicate = board.HasDuplicate();
                if (!hasDuplicate)
                    board.AddTileToRandomLocation(SpawnRepeatTile());
                else
                    board.AddTileToRandomLocation(GenerateTile());
            }
        }

        private Tile GenerateTile()
        {
            return hasDuplicate ? GenerateRandomBoundedTile() : SpawnRepeatTile();
        }

        private Tile SpawnRepeatTile()
        {
            if (board.Empty()) return GenerateRandomBoundedTile();
            var tileList = board.GetTileList();
            var randTile = tileList[rand.Next(0, board.TileCount())];
            return new Tile(Math.Min(randTile.GetValue(), maxTileValue));
        }

        //protected for testability
        protected virtual Tile GenerateRandomBoundedTile()
        {   
            return new Tile(rand.Next(minTileValue, maxTileValue + 1));
        }

        public void SetMaxSpawnTileValue(int value)
        {
            maxTileValue = value;
        }

        public void SetMinSpawnTileValue(int value)
        {
            minTileValue = value;
        }
    }
}