﻿using System;

namespace Tile16_Decoupled
{
    public class WaveController : IWaveController
    {
        protected TileSpawner tileSpawner;
        private Board board;

        //Game rule adjustable fields
        private int maxTileSpawnCount = 4;
        private int biggestTileValue;

        protected WinConditions winConditions;

        public int MaxTileSpawnCount
        {
            get { return maxTileSpawnCount; }
            set
            {
                if (value < 1)
                    throw new InvalidMaxTileSpawnCountException();
                maxTileSpawnCount = value;
            }
        }

        public void SetWinConditions(WinConditions winConditions)
        {
            this.winConditions = winConditions;
        }

        public void SetBoard(Board board)
        {
            this.board = board;
        }

        public void InitializeNewGame()
        {
            tileSpawner = new TileSpawner(board);
            winConditions.Reset();
            biggestTileValue = 0;
        }

        private void SpawnTileClampedByBiggestTileValue()
        {
            biggestTileValue = board.GetBiggestTileValue();
            MinMaxTileValue();
            tileSpawner.Spawn(GenerateSpawnCount());
        }

        private void MinMaxTileValue()
        {
            tileSpawner.SetMaxSpawnTileValue(Math.Max(biggestTileValue - 3, 1));
            tileSpawner.SetMinSpawnTileValue(Math.Max(biggestTileValue - 10, 1));
        }

        private bool NoWinningTileValueAndBoardIsNotFull()
        {
            return winConditions.NotWinningButNotGameOver();
        }

        //Protected for testability
        protected virtual int GenerateSpawnCount()
        {
            Random rand = new Random();
            return rand.Next(2, maxTileSpawnCount + 1);
        }

        public bool NextWave()
        {
            winConditions.UpdateStatusByBiggestTileValue();
            var isGameNotEnded = NoWinningTileValueAndBoardIsNotFull();
            if (isGameNotEnded)
                SpawnTileClampedByBiggestTileValue();
            return isGameNotEnded;
        }
    }

    public class InvalidMaxTileSpawnCountException : Exception
    {
    }
}