﻿using Tile16_Decoupled;

namespace Tile16_Unity_Testable
{
    public class GameController
    {
        private GameControllerMediator mediator;
        public Board GameBoard;
        public static float WaveTimeOut = 5f;
        public static float Time;
        private IWaveController waveController;
        private IWinConditions winCondition;
        private bool pauseTimer;

        public void SetMediator(GameControllerMediator mediator)
        {
            this.mediator = mediator;
            mediator.Initialize();
        }

        public void NewGame()
        {
            TileController.ClearAllTiles();
            TileController.Enabled = true;
            mediator.DisableLoseAnimation();
            mediator.DisableWinAnimation();

            GameBoard = new Board(4,4);
            GlobalTileMover.tileMover = new TileMover(GameBoard);

            Time = 0;
            //mediator.UpdateBoard();
        }

        public void Update()
        {
            if (winCondition.GameOver) return;
            if (!winCondition.IsGameWon())
            {
                CountdownTime();
                AdvanceToNextWave();
            }
            else
                Win();
        }

        private void Win()
        {
            mediator.PlayWinSfx();
            mediator.EnableWinAnimation();
            TileController.Enabled = false;
        }

        private void AdvanceToNextWave()
        {
            if (Time < 0.0001 && Time > -0.0001 || !GameBoard.HasDuplicate())
            {
                TileController.RestoreTileIfPickedUp();
                NextWave();
                Time = WaveTimeOut;
            }
        }

        private void NextWave()
        {
            if (waveController.NextWave())
            {
                mediator.PlayNextWaveSfx();
                mediator.UpdateBoard();
            }
            else
                GameOverButDidntWin();
        }

        private void GameOverButDidntWin()
        {
            if (!winCondition.GameOverButDidntWin()) return;
            mediator.PlayLoseSfx();
            mediator.EnableLoseAnimation();
            TileController.Enabled = false;
        }

        private void CountdownTime()
        {
            if (Time > 0)
            {
                Time = pauseTimer ? Time : Time - mediator.GetDeltaTime();
                if (Time < 0)
                    Time = 0;
            }
        }

        public void SetWaveController(IWaveController waveController)
        {
            if (waveController == null) return;
            this.waveController = waveController;
        }

        public void SetWinCondition(IWinConditions winConditions)
        {
            this.winCondition = winConditions;
        }

        public void PauseTimer()
        {
            pauseTimer = !pauseTimer;
        }
    }
}